"""
$RCSfile: TestCacheManager.py,v $
Unit tests for XMLTransform product

Author: <a href="mailto:cstrong@arielpartners.com">Craeg Strong</a>
Release: 1.0
"""

__cvstag__  = '$Name:  $'[6:-2]
__date__    = '$Date: 2002/08/27 22:57:56 $'[6:-2]
__version__ = '$Revision: 1.11 $'[10:-2]

import os, sys, time

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

# Load fixture
from Testing import ZopeTestCase

# XMLTransform
from Products.XMLTransform import CacheManager 
from Products.XMLTransform import TransformerRegistry
from Products.XMLTransform import XMLTransform

ZopeTestCase.installProduct('XMLTransform')

class CacheManagerTestCase(ZopeTestCase.ZopeTestCase):
    """

    Unit tests for the Cache Manager within the XMLTransform Zope
    Product.  These tests use the ZopeTestCase enhanced testing
    product.

    """

    ################################################################
    # Fixture
    ################################################################

    def afterSetUp(self):
        '''Add object to default fixture'''
        pass

    def afterClear(self):
        '''Clean up after myself'''
        pass

    ################################################################
    # Test Cases
    ################################################################

    def test_01(self):
        "Ensure that cache manager can turn caching on/off in a batch"
        reg = TransformerRegistry.addInstance(self.folder, 'registry')
        
        xsltFile     = open('testfiles/simple.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('aTransformer', '', xsltContents)
        
        xmlFile      = open('testfiles/simple.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('aSource', '', xmlContents)

        xform = XMLTransform.addInstance(self.folder,id='aXMLTransform',title='',description='',
                                         transformerName='aTransformer',sourceID='aSource')
        
        xform2 = XMLTransform.addInstance(self.folder, id='aXMLTransform2',title='',description='',
                                          transformerName='aTransformer',sourceID='aSource')
        
        xform3 = XMLTransform.addInstance(self.folder, id='aXMLTransform3',title='',description='',
                                          transformerName='aTransformer',sourceID='aSource')
        
        self.failUnless(xform.isCachingOn())
        self.failUnless(xform2.isCachingOn())
        self.failUnless(xform3.isCachingOn()        )
        
        cm = CacheManager.addInstance(self.folder,'cm')
        cm.batchSetCachingOff(self.app.REQUEST)
        
        self.failIf(xform.isCachingOn())
        self.failIf(xform2.isCachingOn())
        self.failIf(xform3.isCachingOn()        )
        
        cm.batchSetCachingOn(self.app.REQUEST)
        self.failUnless(xform.isCachingOn())
        self.failUnless(xform2.isCachingOn())
        self.failUnless(xform3.isCachingOn()        )

        # cleanup
        cm.clearCache()

    def test_02(self):
        "Ensure basic caching"
        reg = TransformerRegistry.addInstance(self.folder,'registry')
        
        xsltFile     = open('testfiles/simple.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('aTransformer', '', xsltContents)
        
        xmlFile      = open('testfiles/simple.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('aSource', '', xmlContents)
        
        xform = XMLTransform.addInstance(self.folder, id='xform',title='',description='',
                                         transformerName='aTransformer',sourceID='aSource')
        self.failUnless(xform.isCachingOn())
        
        cm = CacheManager.addInstance(self.folder, 'cm')
        cm.cachePrefix = os.path.join('testfiles', 'Test')
        
        self.assertEquals(xform(xform,self.app.REQUEST), "Hello, world")
        
        cacheFileTS = cm.cacheFileTimeStamp(xform)
        
        cacheFile = os.path.join('testfiles','Test_testFolder_1__xform')
        self.failUnless(os.path.exists(cacheFile))
        files = cm.listCacheFiles()
        self.failUnless(cacheFile in files)
        
        self.assertEquals(cacheFileTS, os.path.getmtime(cacheFile))

        # cleanup
        cm.clearCache()
        self.failIf(os.path.exists(cacheFile))
        
    def test_03(self):
        "Ensure that cache manager can cache multiple xforms in a batch"
        reg = TransformerRegistry.addInstance(self.folder,'registry')
        
        xsltFile     = open('testfiles/simple.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('aTransformer', '', xsltContents)
        
        xmlFile      = open('testfiles/simple.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('aSource', '', xmlContents)
        
        xform = XMLTransform.addInstance(self.folder,id='xform',title='',description='',
                                         transformerName='aTransformer',sourceID='aSource')
        
        xform2 = XMLTransform.addInstance(self.folder,id='xform2',title='',description='',
                                          transformerName='aTransformer',sourceID='aSource')
        
        xform3 = XMLTransform.addInstance(self.folder,id='xform3',title='',description='',
                                          transformerName='aTransformer',sourceID='aSource')
        
        self.failUnless(xform.isCachingOn())
        self.failUnless(xform2.isCachingOn())
        self.failUnless(xform3.isCachingOn()        )
        
        cm = CacheManager.addInstance(self.folder,'cm')
        cm.cachePrefix = os.path.join('testfiles', 'Test')
        cm.regenerateAll(self.app.REQUEST)
        
        cacheFile = os.path.join('testfiles','Test_testFolder_1__xform')
        self.failUnless(os.path.exists(cacheFile))
        cacheFile = os.path.join('testfiles','Test_testFolder_1__xform2')
        self.failUnless(os.path.exists(cacheFile))
        cacheFile = os.path.join('testfiles','Test_testFolder_1__xform3')
        self.failUnless(os.path.exists(cacheFile))
        
        files = cm.listCacheFiles()
        self.failUnless(cacheFile in files)

        # cleanup
        cm.clearCache()
        self.failIf(os.path.exists(cacheFile))
        
    def test_04(self):
        "Ensure that cache manager can recover when cache files are deleted by external party"
        reg = TransformerRegistry.addInstance(self.folder,'registry')
        
        xsltFile     = open('testfiles/simple.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('aTransformer', '', xsltContents)
        
        xmlFile      = open('testfiles/simple.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('aSource', '', xmlContents)
        
        xform = XMLTransform.addInstance(self.folder,id='xform',title='',description='',
                                         transformerName='aTransformer',sourceID='aSource')
        
        xform2 = XMLTransform.addInstance(self.folder,id='xform2',title='',description='',
                                          transformerName='aTransformer',sourceID='aSource')

        self.failUnless(xform.isCachingOn())
        self.failUnless(xform2.isCachingOn())
        
        cm = CacheManager.addInstance(self.folder,'cm')
        cm.cachePrefix = os.path.join('testfiles', 'Test')
        cm.regenerateAll(self.app.REQUEST)
        
        cacheFile = os.path.join('testfiles','Test_testFolder_1__xform')
        self.failUnless(os.path.exists(cacheFile))
        cacheFile = os.path.join('testfiles','Test_testFolder_1__xform2')
        self.failUnless(os.path.exists(cacheFile))
        
        # run it a few times, should get from the cache each time.
        self.assertEquals(xform(xform, self.app.REQUEST), "Hello, world")
        self.assertEquals(xform(xform, self.app.REQUEST), "Hello, world")
        self.assertEquals(xform(xform, self.app.REQUEST), "Hello, world")
        
        cacheFile = os.path.join('testfiles','Test_testFolder_1__xform')
        os.unlink(cacheFile)
        
        # gone from cache, but should not matter
        self.assertEquals(xform(xform, self.app.REQUEST), "Hello, world")        
        
        cacheFile = os.path.join('testfiles','Test_testFolder_1__xform')
        os.unlink(cacheFile)
        cacheFile = os.path.join('testfiles','Test_testFolder_1__xform2')
        os.unlink(cacheFile)
        
        # cleanup
        message = cm.clearCache()
        # message is ignorable        
        self.failIf(message.find("not removed, error") == -1)
        
    def test_05(self):
        "Ensure that removing cache manager causes no errors"
        reg = TransformerRegistry.addInstance(self.folder,'registry')
        
        xsltFile     = open('testfiles/simple.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('aTransformer', '', xsltContents)
        
        xmlFile      = open('testfiles/simple.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('aSource', '', xmlContents)
        
        xform = XMLTransform.addInstance(self.folder,id='xform',title='',description='',
                                         transformerName='aTransformer',sourceID='aSource')
        
        xform2 = XMLTransform.addInstance(self.folder,id='xform2',title='',description='',
                                          transformerName='aTransformer',sourceID='aSource')

        self.failUnless(xform.isCachingOn())
        self.failUnless(xform2.isCachingOn())
        
        cm = CacheManager.addInstance(self.folder,'cm')
        cm.cachePrefix = os.path.join('testfiles', 'Test')
        
        self.assertEquals(xform(xform, self.app.REQUEST), "Hello, world")
        self.assertEquals(xform2(xform2, self.app.REQUEST), "Hello, world")

        # do it again, for good measure. Should get from the cache now
        self.assertEquals(xform(xform, self.app.REQUEST), "Hello, world")
        self.assertEquals(xform2(xform2, self.app.REQUEST), "Hello, world")

        self.failUnless(xform.findCacheManager())
        self.failUnless(xform2.findCacheManager())

        cm.clearCache()

        # remove the cache.  make sure it is truly dead.
        self.folder._delObject('cm')
        del cm
        cm = None

        # should not be a problem, cache manager is optional
        self.assertEquals(xform(xform, self.app.REQUEST), "Hello, world")
        self.assertEquals(xform2(xform2, self.app.REQUEST), "Hello, world")

        self.failIf(xform.findCacheManager())
        self.failIf(xform2.findCacheManager())

    def test_05(self):
        "Ensure two objects with same ID, in different folders, can be cached"
        reg = TransformerRegistry.addInstance(self.folder,'registry')
        
        xsltFile     = open('testfiles/simple.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('aTransformer', '', xsltContents)
        
        xmlFile      = open('testfiles/simple.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('aSource', '', xmlContents)
        
        self.folder.manage_addFolder('folder1')
        folder1 = self.folder['folder1']
        xform1 = XMLTransform.addInstance(folder1,id='xform1',title='',description='',
                                          transformerName='aTransformer',sourceID='aSource')        
        
        self.folder.manage_addFolder('folder2')
        folder2 = self.folder['folder2']
        xform2 = XMLTransform.addInstance(folder2,id='xform2',title='',description='',
                                          transformerName='aTransformer',sourceID='aSource')
        self.failUnless(xform1.isCachingOn())
        self.failUnless(xform2.isCachingOn())
        
        cm = CacheManager.addInstance(self.folder,'cm')
        cm.cachePrefix = os.path.join('testfiles', 'Test')
        
        self.assertEquals(xform1(xform1,self.app.REQUEST), "Hello, world")
        self.assertEquals(xform2(xform2,self.app.REQUEST), "Hello, world")        
        
        cacheFile = os.path.join('testfiles','Test_testFolder_1__folder1_xform1')
        self.failUnless(os.path.exists(cacheFile))
        files = cm.listCacheFiles()
        self.failUnless(cacheFile in files)
        
        cacheFile = os.path.join('testfiles','Test_testFolder_1__folder2_xform2')
        self.failUnless(os.path.exists(cacheFile))
        files = cm.listCacheFiles()
        self.failUnless(cacheFile in files)
        
        # cleanup
        cm.clearCache()

    def test_06(self):
        "Ensure that changing to a different transformer invalidates the cache"

        reg = TransformerRegistry.addInstance(self.folder,'registry')
        
        xsltFile     = open('testfiles/simple.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('tran1', '', xsltContents)

        xsltFile     = open('testfiles/alternate.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('tran2', '', xsltContents)

        xmlFile      = open('testfiles/simple.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('aSource', '', xmlContents)
        
        xform = XMLTransform.addInstance(self.folder,id='xform',title='',description='',
                                         transformerName='tran1',sourceID='aSource')
        self.failUnless(xform.isCachingOn())
        
        cm = CacheManager.addInstance(self.folder,'cm')
        cm.cachePrefix = os.path.join('testfiles', 'Test')
        
        self.assertEquals(xform(xform,self.app.REQUEST), "Hello, world")
        # do it again, for good measure
        self.assertEquals(xform(xform,self.app.REQUEST), "Hello, world")        

        # now change the transformer
        xform.editTransform('tran2', 'aSource')
        self.assertEquals(xform(xform,self.app.REQUEST), "Hello, world and goodbye")

    def test_07(self):
        "Ensure that changing to a different XML doc invalidates the cache"

        reg = TransformerRegistry.addInstance(self.folder,'registry')
        
        xsltFile     = open('testfiles/simple.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('tran', '', xsltContents)

        xmlFile      = open('testfiles/simple.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('src1', '', xmlContents)

        xmlFile      = open('testfiles/param.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('src2', '', xmlContents)

        xform = XMLTransform.addInstance(self.folder,id='xform',title='',description='',
                                         transformerName='tran',sourceID='src1')
        self.failUnless(xform.isCachingOn())
        
        cm = CacheManager.addInstance(self.folder,'cm')
        cm.cachePrefix = os.path.join('testfiles', 'Test')
        
        self.assertEquals(xform(xform,self.app.REQUEST), "Hello, world")
        # do it again, for good measure
        self.assertEquals(xform(xform,self.app.REQUEST), "Hello, world")        

        self.assertEquals(xform.isCacheFileUpToDate(), 1)
        outOfDaters = cm.listOutOfDate()
        self.failIf('xform' in outOfDaters)
        upToDaters = cm.listUpToDate()
        self.failUnless('xform' in upToDaters)

        # now change the XML document
        xform.editTransform('tran', 'src2')
        
        self.assertEquals(xform.isCacheFileUpToDate(), 0)
        outOfDaters = cm.listOutOfDate()
        self.failUnless('xform' in outOfDaters)
        upToDaters = cm.listUpToDate()
        self.failIf('xform' in upToDaters)
        
        self.assertEquals(xform(xform,self.app.REQUEST), "Hello, ")

        # cleanup
        cm.clearCache()

    def test_08(self):
        "Ensure that editing the source XML doc invalidates the cache"

        reg = TransformerRegistry.addInstance(self.folder,'registry')
        
        xsltFile     = open('testfiles/simple.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('tran', '', xsltContents)

        xmlFile      = open('testfiles/simple.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('src', '', xmlContents)
        src = self.folder['src']

        xform = XMLTransform.addInstance(self.folder,id='xform',title='',description='',
                                         transformerName='tran',sourceID='src')
        self.failUnless(xform.isCachingOn())
        
        cm = CacheManager.addInstance(self.folder,'cm')
        cm.cachePrefix = os.path.join('testfiles', 'Test')
        
        self.assertEquals(xform(xform,self.app.REQUEST), "Hello, world")
        # do it again, for good measure
        self.assertEquals(xform(xform,self.app.REQUEST), "Hello, world")        

        # now edit the contents of the XML document First we have to
        # wait a little bit.  Sleep for 2 seconds to ensure we get a
        # different number.
        time.sleep(2)
        # Change the contents
        src.manage_edit("<?xml version=\"1.0\"?><!DOCTYPE Document><Document>Hello, zope</Document>", "My Title")
        self.assertEquals(xform(xform,self.app.REQUEST), "Hello, zope")

        # cleanup
        cm.clearCache()

    def test_08(self):
        "Ensure that editing the transformer invalidates the cache"

        reg = TransformerRegistry.addInstance(self.folder,'registry')
        
        xsltFile     = open('testfiles/simple.xsl','rb')
        xsltContents = xsltFile.read()
        reg.manage_addDTMLMethod('tran', '', xsltContents)
        tran = reg['tran']
        
        xmlFile      = open('testfiles/simple.xml', 'rb')
        xmlContents  = xmlFile.read()
        self.folder.manage_addDTMLMethod('src', '', xmlContents)

        xform = XMLTransform.addInstance(self.folder,id='xform',title='',description='',
                                         transformerName='tran',sourceID='src')
        self.failUnless(xform.isCachingOn())
        
        cm = CacheManager.addInstance(self.folder,'cm')
        cm.cachePrefix = os.path.join('testfiles', 'Test')
        
        self.assertEquals(xform(xform,self.app.REQUEST), "Hello, world")
        # do it again, for good measure
        self.assertEquals(xform(xform,self.app.REQUEST), "Hello, world")        

        # now edit the contents of the transformer document.  First we
        # have to wait a little bit.  Sleep for 2 seconds to ensure we
        # get a different number.
        time.sleep(2)
        # Change the contents
        xsltFile     = open('testfiles/alternate.xsl','rb')
        xsltContents = xsltFile.read()
        
        tran.manage_edit(xsltContents, "My Title")
        self.assertEquals(xform(xform,self.app.REQUEST), "Hello, world and goodbye")

        # cleanup
        cm.clearCache()

    ################################################################
    # Test Runner Setup
    ################################################################
            
if __name__ == '__main__':
    framework(descriptions=1, verbosity=2)
else:
    # While framework.py provides its own test_suite() 
    # method the testrunner utility does not.
    import unittest
    def test_suite():
        suite = unittest.TestSuite()
        suite.addTest(unittest.makeSuite(CacheManagerTestCase))
        return suite
