"""
$RCSfile: TestXSLTProcessor.py,v $
Unit tests for XMLTransform product

Author: <a href="mailto:cstrong@arielpartners.com">Craeg Strong</a>
Release: 1.0
"""

__cvstag__  = '$Name:  $'[6:-2]
__date__    = '$Date: 2002/08/26 04:44:26 $'[6:-2]
__version__ = '$Revision: 1.13 $'[10:-2]

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

# Load fixture
from Testing import ZopeTestCase

ZopeTestCase.installProduct('XMLTransform')

# XMLTransform
from Products.XMLTransform.XMLTransform import XSLTProcessor

class XSLTProcessorTestCase(ZopeTestCase.ZopeTestCase):
    """

    XSLT Processor tests.  This is particularly useful if switching
    from one library to another, for example upgrading 4Suite or
    switching to Pyana or Saxon.

    """

    ################################################################
    # Fixture
    ################################################################

    def afterSetUp(self):
        '''Add object to default fixture'''
        self.prod = self.folder.manage_addProduct['XMLTransform']
        self.processor = XSLTProcessor()
        self.processor.setDebugLevel(0)

    def afterClear(self):
        '''Clean up after myself'''
        try: del self.prod
        except AttributeError: pass

    ################################################################
    # Test Cases
    ################################################################

    def test_01(self):
        "Basic processor tests"

        xmlFile      = open('testfiles/simple.xml', 'rb')
        xmlContents  = xmlFile.read()
        xsltFile     = open('testfiles/simple.xsl','rb')
        xsltContents = xsltFile.read()

        result = self.processor.transformGuts(xmlContents, xsltContents,
                                              "", None, None, None)

        self.assertEquals(result, "Hello, world")

    def test_02(self):
        "Test for Invalid XML"

        xmlFile      = open('testfiles/invalid.xml', 'rb')
        xmlContents  = xmlFile.read()
        xsltFile     = open('testfiles/simple.xsl','rb')
        xsltContents = xsltFile.read()

        self.assertRaises(Exception, self.processor.transformGuts,
                          xmlContents, xsltContents,
                          "", None, None, None)

    def test_03(self):
        "Test for non valid xslt transformer"

        xmlFile      = open('testfiles/simple.xml', 'rb')
        xmlContents  = xmlFile.read()
        xsltFile     = open('testfiles/invalid.xsl','rb')
        xsltContents = xsltFile.read()

        self.assertRaises(Exception, self.processor.transformGuts,
                          xmlContents, xsltContents,
                          "", None, None, None)

    def test_04(self):
        "Test that the processor handles parameters correctly"

        xmlFile      = open('testfiles/param.xml', 'rb')
        xmlContents  = xmlFile.read()
        xsltFile     = open('testfiles/param.xsl','rb')
        xsltContents = xsltFile.read()

        params = {}
        self.processor.addParam(params, "who", "world")
        self.processor.addParam(params, "message", "goodbye")

        result = self.processor.transformGuts(xmlContents, xsltContents,
                                              "",None, params, None)

        self.assertEquals(result, "Hello, world and goodbye")

    ################################################################
    # Test Runner Setup
    ################################################################

if __name__ == '__main__':
    framework(descriptions=1, verbosity=2)
else:
    # While framework.py provides its own test_suite() 
    # method the testrunner utility does not.
    import unittest
    def test_suite():
        suite = unittest.TestSuite()
        suite.addTest(unittest.makeSuite(XSLTProcessorTestCase))
        return suite
